@@ -3,7 +3,7 @@
 **	DMARC_DNS_GET_RECORD -- looks up and returns the txt record
 **	DMARC_DNS_TEST_RECORD -- hook to test
 ** 
-**  Copyright (c) 2012-2015, The Trusted Domain Project.  All rights reserved.
+**  Copyright (c) 2012-2016, The Trusted Domain Project.  All rights reserved.
 ************************************************************************/ 
 
 #ifndef BIND_8_COMPAT
@@ -217,7 +217,7 @@ dmarc_dns_get_record(char *domain, int *reply, char *got_txtbuf, size_t got_txtl
 	_res.options |= RES_USE_DNSSEC;
 #endif
 	(void) opendmarc_policy_library_dns_hook(&_res.nscount,
-                                                 &_res.nsaddr_list);
+                                                 _res.nsaddr_list);
 	answer_len = res_query(bp, C_IN, T_TXT, answer_buf, sizeof answer_buf);
 #endif /* HAVE_RES_NINIT */
 	if (answer_len < 0)
@@ -316,8 +316,8 @@ dmarc_dns_get_record(char *domain, int *reply, char *got_txtbuf, size_t got_txtl
 #ifdef T_RRSIG
 		else if (type == T_RRSIG)
 		{
-			GETSHORT(answerlen, cur_ptr);
-			cur_ptr += answerlen;
+			GETSHORT(answer_len, cur_ptr);
+			cur_ptr += answer_len;
 		}
 #endif /* T_RRSIG */
 		else if (type != T_TXT)
