@@ -1,7 +1,7 @@
 /*************************************************************************
 ** The user interface to the rest of this library.
 **
-**  Copyright (c) 2012-2015, The Trusted Domain Project.  All rights reserved.
+**  Copyright (c) 2012-2016, The Trusted Domain Project.  All rights reserved.
 **************************************************************************/
 
 #include "opendmarc_internal.h"
@@ -111,7 +111,8 @@ opendmarc_policy_connect_init(u_char *ip_addr, int is_ipv6)
 		return NULL;
 	}
 	(void) memset(pctx, '\0', sizeof(DMARC_POLICY_T));
-	pctx->p   = DMARC_RECORD_P_UNSPECIFIED;
+	pctx->p = DMARC_RECORD_P_UNSPECIFIED;
+	pctx->sp = DMARC_RECORD_P_UNSPECIFIED;
 	pctx->ip_addr = (u_char *)strdup((char *)ip_addr);
 	if (pctx->ip_addr == NULL)
 	{
@@ -491,11 +492,12 @@ opendmarc_policy_store_dkim(DMARC_POLICY_T *pctx, u_char *d_equal_domain, int dk
 	}
 
 	/*
-	 * See if the d= us a superset of the from domain.
+	 * See if the d= is a superset of the from domain.
 	 * If so and if we have not already found
 	 * a best match, make this the temporary best match.
 	 */
-	if (opendmarc_policy_check_alignment(dp, pctx->from_domain, pctx->adkim))
+	if (opendmarc_policy_check_alignment(dp, pctx->from_domain,
+	                                     pctx->adkim) == 0)
 	{
 		if (pctx->dkim_domain != NULL)
 		{
@@ -833,6 +835,21 @@ opendmarc_get_policy_to_enforce(DMARC_POLICY_T *pctx)
 	    pctx->dkim_alignment == DMARC_POLICY_DKIM_ALIGNMENT_PASS)
 		return DMARC_POLICY_PASS;
 
+	if (pctx->organizational_domain != NULL)
+	{
+		switch (pctx->sp)
+		{
+		  case DMARC_RECORD_P_REJECT:
+			return DMARC_POLICY_REJECT;
+
+		  case DMARC_RECORD_P_QUARANTINE:
+			return DMARC_POLICY_QUARANTINE;
+
+		  case DMARC_RECORD_P_NONE:
+			return DMARC_POLICY_NONE;
+		}
+	}
+
 	switch (pctx->p)
 	{
 		case DMARC_RECORD_P_REJECT:
@@ -1466,6 +1483,29 @@ opendmarc_policy_fetch_utilized_domain(DMARC_POLICY_T *pctx, u_char *buf, size_t
 	return DMARC_PARSE_OKAY;
 }
 
+/**************************************************************************
+** OPENDMARC_GET_POLICY_TOKEN_USED -- Which policy was actually used
+**
+**	Parameters:
+**		pctx	-- A Policy context
+**	Returns:
+**		DMARC_PARSE_ERROR_NULL_CTX	-- pctx == NULL
+**		DMARC_USED_POLICY_IS_P		-- Domain policy is used
+**		DMARC_USED_POLICY_IS_SP		-- Sub-domain policy is used
+***************************************************************************/
+OPENDMARC_STATUS_T
+opendmarc_get_policy_token_used(DMARC_POLICY_T *pctx)
+{
+
+	if (pctx == NULL)
+		return DMARC_PARSE_ERROR_NULL_CTX;
+	if (pctx->organizational_domain != NULL &&
+	    pctx->sp != DMARC_RECORD_P_UNSPECIFIED)
+		return DMARC_USED_POLICY_IS_SP;
+	else
+		return DMARC_USED_POLICY_IS_P;
+}
+
 /******************************************************************************
 ** OPENDMARC_POLICY_LIBRARY_DNS_HOOK -- Internal hook for dmarc_dns_get_record
 *******************************************************************************/
@@ -1474,7 +1514,6 @@ opendmarc_policy_library_dns_hook(int *nscountp,
                                   struct sockaddr_in *nsaddr_list)
 {
 	int i;
-	struct sockaddr_in *ol = Opendmarc_Libp->nsaddr_list;
 
 	if (nscountp == NULL || nsaddr_list == NULL)
 		return;
@@ -1484,9 +1523,7 @@ opendmarc_policy_library_dns_hook(int *nscountp,
 		return;
 	for (i = 0; i < Opendmarc_Libp->nscount; i++)
 	{
-		(void) memcpy(&ol[i],
-		              &nsaddr_list[i],
-                              sizeof(struct sockaddr_in));
+		nsaddr_list[i] = Opendmarc_Libp->nsaddr_list[i];
 	}
 	*nscountp = i;
 	return;
@@ -1820,15 +1857,16 @@ opendmarc_policy_to_buf(DMARC_POLICY_T *pctx, char *buf, size_t buflen)
 	{
 		if (strlcat(buf, "UNSPECIFIED", buflen) >= buflen) return E2BIG;
 	}
-	if ((pctx->rf|DMARC_RECORD_RF_AFRF) != 0)
+	if ((pctx->rf&DMARC_RECORD_RF_AFRF) != 0)
 	{
 		if (strlcat(buf, "AFRF", buflen) >= buflen) return E2BIG;
 	}
-	if ((pctx->rf|DMARC_RECORD_RF_IODEF) != 0 && (pctx->rf|DMARC_RECORD_RF_AFRF) != 0)
+	if ((pctx->rf&DMARC_RECORD_RF_IODEF) != 0 &&
+	    (pctx->rf&DMARC_RECORD_RF_AFRF) != 0)
 	{
 		if (strlcat(buf, ",", buflen) >= buflen) return E2BIG;
 	}
-	if ((pctx->rf|DMARC_RECORD_RF_IODEF) != 0)
+	if ((pctx->rf&DMARC_RECORD_RF_IODEF) != 0)
 	{
 		if (strlcat(buf, "IODEF", buflen) >= buflen) return E2BIG;
 	}
@@ -1866,19 +1904,19 @@ opendmarc_policy_to_buf(DMARC_POLICY_T *pctx, char *buf, size_t buflen)
 	{
 		if (strlcat(buf, "UNSPECIFIED", buflen) >= buflen) return E2BIG;
 	}
-	if ((pctx->fo|DMARC_RECORD_FO_0) != 0)
+	if ((pctx->fo&DMARC_RECORD_FO_0) != 0)
 	{
 		if (strlcat(buf, "0:", buflen) >= buflen) return E2BIG;
 	}
-	if ((pctx->fo|DMARC_RECORD_FO_1) != 0)
+	if ((pctx->fo&DMARC_RECORD_FO_1) != 0)
 	{
 		if (strlcat(buf, "1:", buflen) >= buflen) return E2BIG;
 	}
-	if ((pctx->fo|DMARC_RECORD_FO_D) != 0)
+	if ((pctx->fo&DMARC_RECORD_FO_D) != 0)
 	{
 		if (strlcat(buf, "d:", buflen) >= buflen) return E2BIG;
 	}
-	if ((pctx->fo|DMARC_RECORD_FO_S) != 0)
+	if ((pctx->fo&DMARC_RECORD_FO_S) != 0)
 	{
 		if (strlcat(buf, "s:", buflen) >= buflen) return E2BIG;
 	}
