@@ -113,6 +113,7 @@ opendmarc_spf2_specify_ip_address(SPF_CTX_T *spfctx, char *ip_address, size_t ip
 	 * we don't care at this point if it is ipv6 or ipv4
 	 */
 	SPF_request_set_ipv4_str(spfctx->spf_request, ip_address);
+	SPF_request_set_ipv6_str(spfctx->spf_request, ip_address);
 	return 0;
 }
 
@@ -163,10 +164,10 @@ opendmarc_spf2_test(char *ip_address, char *mail_from_domain, char *helo_domain,
 	}
 
 	ret = opendmarc_spf2_find_mailfrom_domain(ctx, mail_from_domain, mfrom, sizeof mfrom, used_mfrom);
-	if (ret |= 0 || used_mfrom == FALSE)
+	if (ret != 0 || *used_mfrom == FALSE)
 	{
 		(void) strlcpy(helo, helo_domain, sizeof helo);
-		SPF_request_set_env_from(ctx->spf_request, helo);
+		SPF_request_set_helo_dom(ctx->spf_request, helo);
 	}
 	else
 	{
@@ -185,8 +186,10 @@ opendmarc_spf2_test(char *ip_address, char *mail_from_domain, char *helo_domain,
 	{
 		switch (ret)
 		{
-		    case SPF_RESULT_NEUTRAL:
 		    case SPF_RESULT_NONE:
+			return DMARC_POLICY_SPF_OUTCOME_NONE;
+
+		    case SPF_RESULT_NEUTRAL:
 		    case SPF_RESULT_SOFTFAIL:
 			if (softfail_okay_flag == TRUE)
 				return DMARC_POLICY_SPF_OUTCOME_PASS;
@@ -824,6 +827,13 @@ opendmarc_spf_ipv6_cidr_check(char *ipv6_str, char *cidr_string)
 	{
 		int twobyte_mask, tmp_mask;
 
+		if (cidr_bits >= 16)
+		{
+			low_iary.values[i] = 0;
+			hi_iary.values[i] = 0xFFFF;
+			cidr_bits = cidr_bits - 16;
+			continue;
+		}
 		twobyte_mask = cidr_bits % 16;
 		tmp_mask = (0xFFFF << twobyte_mask);
 		low_iary.values[i] = low_iary.values[i] & tmp_mask;
@@ -832,10 +842,9 @@ opendmarc_spf_ipv6_cidr_check(char *ipv6_str, char *cidr_string)
 		hi_iary.values[i]  = hi_iary.values[i] | tmp_mask;
 		if (cidr_bits < 16)
 			break;
-		cidr_bits = cidr_bits / 16;
+		cidr_bits = cidr_bits - 16;
 	}
 
-
 	taghi = FALSE;
 	taglo = FALSE;
 
