@@ -5,7 +5,21 @@
 **  Copyright (c) 2012-2014, The Trusted Domain Project.  All rights reserved.
 ************************************************************************/ 
 # include "opendmarc_internal.h"
-# include "opendmarc_strl.h"
+
+/* libbsd if found */
+#ifdef USE_BSD_H
+# include <bsd/string.h>
+#endif /* USE_BSD_H */
+
+/* libstrl if needed */
+#ifdef USE_STRL_H
+# include <strl.h>
+#endif /* USE_STRL_H */
+
+/* opendmarc_strl if needed */
+#ifdef USE_DMARCSTRL_H
+# include <opendmarc_strl.h>
+#endif /* USE_DMARCSTRL_H */
 
 static char *Taglist[] = {
 	"adkim",
