@@ -1,11 +1,13 @@
 #include "../opendmarc_internal.h"
-#define OPENDMARC_POLICY_C
+#ifndef OPENDMARC_POLICY_C
+# define OPENDMARC_POLICY_C
+#endif /* ! OPENDMARC_POLICY_C */
 #include "../dmarc.h"
 
 int
 main(int argc, char **argv)
 {
-	char * record = "v=DMARC1; p=none; rf=afrf; rua=mailto:dmarc-a@abuse.net; ruf=mailto:dmarc-f@abuse.net";
+	char * record = "v=DMARC1; p=none; rf=afrf; fo=1:s rua=mailto:dmarc-a@abuse.net; ruf=mailto:dmarc-f@abuse.net";
 	int	pass, fails, count;
 	DMARC_POLICY_T *pctx;
 	OPENDMARC_STATUS_T status;
