@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2012-2015, The Trusted Domain Project.  All rights reserved.
+**  Copyright (c) 2012-2017, The Trusted Domain Project.  All rights reserved.
 */
 
 #include "build-config.h"
@@ -17,7 +17,6 @@
 #endif /* __linux__ */
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <sys/file.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -134,7 +133,7 @@ struct dmarcf_connctx
 	char			cctx_host[MAXHOSTNAMELEN + 1];
 #if WITH_SPF
 	char			cctx_helo[MAXHOSTNAMELEN + 1];
-	char			cctx_rawmfrom[MAXHOSTNAMELEN + 1];
+	char			cctx_rawmfrom[BUFRSZ + 1];
 #endif
 };
 typedef struct dmarcf_connctx * DMARCF_CONNCTX;
@@ -395,7 +394,7 @@ dmarcf_getsymval(SMFICTX *ctx, char *sym)
 **  	str -- the value of the Received-SPF field to analyze
 **  	
 **  Return value:
-**  	A DMARC_POLICY_SPF_* constant.
+**  	A ARES_RESULT_* constant.
 */
 
 int
@@ -473,14 +472,19 @@ dmarcf_parse_received_spf(char *str)
 	}
 
 	if (strcasecmp(result, "pass") == 0)
-		return DMARC_POLICY_SPF_OUTCOME_PASS;
-	else if (strcasecmp(result, "fail") == 0 ||
-	         strcasecmp(result, "softfail") == 0)
-		return DMARC_POLICY_SPF_OUTCOME_FAIL;
+		return ARES_RESULT_PASS;
+	else if (strcasecmp(result, "fail") == 0)
+		return ARES_RESULT_FAIL;
+	else if (strcasecmp(result, "softfail") == 0)
+		return ARES_RESULT_SOFTFAIL;
+	else if (strcasecmp(result, "neutral") == 0)
+		return ARES_RESULT_NEUTRAL;
 	else if (strcasecmp(result, "temperror") == 0)
-		return DMARC_POLICY_SPF_OUTCOME_TMPFAIL;
+		return ARES_RESULT_TEMPERROR;
+	else if (strcasecmp(result, "none") == 0)
+		return ARES_RESULT_NONE;
 	else
-		return DMARC_POLICY_SPF_OUTCOME_NONE;
+		return ARES_RESULT_PERMERROR;
 }
 
 /*
@@ -1102,7 +1106,7 @@ dmarcf_init_syslog(char *facility)
 	code = LOG_MAIL;
 	if (facility != NULL)
 	{
-		for (p = log_facilities; p != NULL; p++)
+		for (p = log_facilities; p->str != NULL; p++)
 		{
 			if (strcasecmp(p->str, facility) == 0)
 			{
@@ -1462,8 +1466,8 @@ dmarcf_config_reload(void)
 
 		if (!err && new->conf_pslist != NULL)
 		{
-			if (opendmarc_tld_read_file(new->conf_pslist, "#",
-			                            NULL, NULL) != 0)
+			if (opendmarc_tld_read_file(new->conf_pslist, "//",
+			                            "*.", "!") != 0)
 			{
 				if (curconf->conf_dolog)
 				{
@@ -1585,7 +1589,10 @@ mlfi_negotiate(SMFICTX *ctx,
 {
 	unsigned long reqactions = SMFIF_ADDHDRS|SMFIF_QUARANTINE;
 	unsigned long wantactions = 0;
-	unsigned long protosteps = (SMFIP_NOHELO |
+	unsigned long protosteps = (
+#if !WITH_SPF
+				    SMFIP_NOHELO |
+#endif /* !WITH_SPF */
 	                            SMFIP_NOUNKNOWN |
 	                            SMFIP_NOBODY |
 	                            SMFIP_NODATA |
@@ -1683,7 +1690,11 @@ mlfi_connect(SMFICTX *ctx, char *host, _SOCK_ADDR *ip)
 
 	if (dmarcf_checkhost(host, ignore) ||
 	    (ip != NULL && dmarcf_checkip(ip, ignore)))
+	{
+		if (curconf->conf_dolog)
+			syslog(LOG_INFO, "ignoring connection from %s", host);
 		return SMFIS_ACCEPT;
+	}
 
 	/* copy hostname and IP information to a connection context */
 	cc = dmarcf_getpriv(ctx);
@@ -2018,7 +2029,10 @@ mlfi_eom(SMFICTX *ctx)
 	int result;
 	sfsistat ret = SMFIS_CONTINUE;
 	OPENDMARC_STATUS_T ostatus;
+	OPENDMARC_STATUS_T apused;
+	char *apolicy = NULL;
 	char *aresult = NULL;
+	char *adisposition = NULL;
 	char *hostname = NULL;
 	char *authservid = NULL;
 	char *spfaddr;
@@ -2027,8 +2041,10 @@ mlfi_eom(SMFICTX *ctx)
 	struct dmarcf_config *conf;
 	struct dmarcf_header *hdr;
 	struct dmarcf_header *from;
+	u_char *reqhdrs_error = NULL;
 	u_char *user;
 	u_char *domain;
+	u_char *bang;
 	u_char **ruv;
 	unsigned char header[MAXHEADER + 1];
 	unsigned char addrbuf[BUFRSZ + 1];
@@ -2072,65 +2088,62 @@ mlfi_eom(SMFICTX *ctx)
 	/* select authserv-id to use when generating result headers */
 	authservid = conf->conf_authservid;
 	if (authservid == NULL)
+	{
 		authservid = hostname;
 
-	if (conf->conf_dolog)
-	{
-		syslog(LOG_INFO, "implicit authentication service: %s",
-		       authservid);
+		if (conf->conf_dolog)
+		{
+			syslog(LOG_INFO, "implicit authentication service: %s",
+			       authservid);
+		}
 	}
 
 	/* ensure there was a From field */
 	from = dmarcf_findheader(dfc, "From", 0);
 
-	/* if requested, verify RFC5322-required headers (RFC5322 3.6) */
-	if (conf->conf_reqhdrs)
+	/* verify RFC5322-required headers (RFC5322 3.6) */
+	if (from == NULL ||
+	    dmarcf_findheader(dfc, "From", 1) != NULL)
+		reqhdrs_error = "not exactly one From field";
+
+	if (dmarcf_findheader(dfc, "Date", 0) == NULL ||
+	    dmarcf_findheader(dfc, "Date", 1) != NULL)
+		reqhdrs_error = "not exactly one Date field";
+
+	if (dmarcf_findheader(dfc, "Reply-To", 1) != NULL)
+		reqhdrs_error = "multiple Reply-To fields";
+
+	if (dmarcf_findheader(dfc, "To", 1) != NULL)
+		reqhdrs_error = "multiple To fields";
+
+	if (dmarcf_findheader(dfc, "Cc", 1) != NULL)
+		reqhdrs_error = "multiple Cc fields";
+
+	if (dmarcf_findheader(dfc, "Bcc", 1) != NULL)
+		reqhdrs_error = "multiple Bcc fields";
+
+	if (dmarcf_findheader(dfc, "Message-Id", 1) != NULL)
+		reqhdrs_error = "multiple Message-Id fields";
+
+	if (dmarcf_findheader(dfc, "In-Reply-To", 1) != NULL)
+		reqhdrs_error = "multiple In-Reply-To fields";
+
+	if (dmarcf_findheader(dfc, "References", 1) != NULL)
+		reqhdrs_error = "multiple References fields";
+
+	if (dmarcf_findheader(dfc, "Subject", 1) != NULL)
+		reqhdrs_error = "multiple Subject fields";
+
+	if (conf->conf_reqhdrs && reqhdrs_error != NULL)
 	{
-		unsigned char* reqhdrs_error = NULL; /* no error */
-
-		if (from == NULL ||
-		    dmarcf_findheader(dfc, "From", 1) != NULL)
-			reqhdrs_error = "not exactly one From field";
-
-		if (dmarcf_findheader(dfc, "Date", 0) == NULL ||
-		    dmarcf_findheader(dfc, "Date", 1) != NULL)
-			reqhdrs_error = "not exactly one Date field";
-
-		if (dmarcf_findheader(dfc, "Reply-To", 1) != NULL)
-			reqhdrs_error = "multiple Reply-To fields";
-
-		if (dmarcf_findheader(dfc, "To", 1) != NULL)
-			reqhdrs_error = "multiple To fields";
-
-		if (dmarcf_findheader(dfc, "Cc", 1) != NULL)
-			reqhdrs_error = "multiple Cc fields";
-
-		if (dmarcf_findheader(dfc, "Bcc", 1) != NULL)
-			reqhdrs_error = "multiple Bcc fields";
-
-		if (dmarcf_findheader(dfc, "Message-Id", 1) != NULL)
-			reqhdrs_error = "multiple Message-Id fields";
-
-		if (dmarcf_findheader(dfc, "In-Reply-To", 1) != NULL)
-			reqhdrs_error = "multiple In-Reply-To fields";
-
-		if (dmarcf_findheader(dfc, "References", 1) != NULL)
-			reqhdrs_error = "multiple References fields";
-
-		if (dmarcf_findheader(dfc, "Subject", 1) != NULL)
-			reqhdrs_error = "multiple Subject fields";
-
-		if (reqhdrs_error != NULL)
+		if (conf->conf_dolog)
 		{
-			if (conf->conf_dolog)
-			{
-				syslog(LOG_INFO,
-				       "%s: RFC5322 requirement error: %s",
-				       dfc->mctx_jobid, reqhdrs_error);
-			}
-
-			return SMFIS_REJECT;
+			syslog(LOG_INFO,
+			       "%s: RFC5322 requirement error: %s",
+			       dfc->mctx_jobid, reqhdrs_error);
 		}
+
+		return SMFIS_REJECT;
 	}
 
 	/* if there was no From:, there's nothing to process past here */
@@ -2469,6 +2482,33 @@ mlfi_eom(SMFICTX *ctx)
 				dmarcf_dstring_printf(dfc->mctx_histbuf,
 				                      "spf %d\n", spfres);
 
+				dfc->mctx_spfresult = spfres;
+
+				switch (dfc->mctx_spfresult)
+				{
+				    case ARES_RESULT_PASS:
+					spfres = DMARC_POLICY_SPF_OUTCOME_PASS;
+					break;
+
+				    case ARES_RESULT_NONE:
+					spfres = DMARC_POLICY_SPF_OUTCOME_NONE;
+					break;
+
+				    case ARES_RESULT_TEMPERROR:
+					spfres = DMARC_POLICY_SPF_OUTCOME_TMPFAIL;
+					break;
+
+				    case ARES_RESULT_FAIL:
+				    case ARES_RESULT_NEUTRAL:
+				    case ARES_RESULT_SOFTFAIL:
+					spfres = DMARC_POLICY_SPF_OUTCOME_FAIL;
+					break;
+
+				    default: /* e.g. ARES_RESULT_PERMERROR */
+					spfres = DMARC_POLICY_SPF_OUTCOME_NONE;
+					break;
+				}
+
 				/* use the MAIL FROM domain */
 				ostatus = opendmarc_policy_store_spf(cc->cctx_dmarc,
 				                                     dfc->mctx_envdomain,
@@ -2524,7 +2564,7 @@ mlfi_eom(SMFICTX *ctx)
 				                                     spf_result,
 				                                     spf_mode,
 				                                     human);
-			switch (spf_mode)
+			switch (spf_result)
 			{
 			    case DMARC_POLICY_SPF_OUTCOME_PASS:
 				pass_fail = "pass";
@@ -2576,6 +2616,23 @@ mlfi_eom(SMFICTX *ctx)
 					       AUTHRESULTSHDR);
 				}
 			}
+
+			if (conf->conf_dolog)
+			{
+				char *mode;
+
+				if (spf_mode == DMARC_POLICY_SPF_ORIGIN_HELO)
+					mode = "helo";
+				else
+					mode = "mailfrom";
+
+				syslog(LOG_INFO,
+				       "%s: SPF(%s): %s %s",
+				       dfc->mctx_jobid,
+				       mode,
+				       use_domain,
+				       pass_fail);
+			}
 		}
 #endif /* WITH_SPF */
 
@@ -2673,6 +2730,25 @@ mlfi_eom(SMFICTX *ctx)
 	                      align_dkim);
 	dmarcf_dstring_printf(dfc->mctx_histbuf, "align_spf %d\n", align_spf);
 
+	/* prepare human readable policy string for later processing */
+	apused = opendmarc_get_policy_token_used(cc->cctx_dmarc);
+	switch (apused == DMARC_USED_POLICY_IS_SP ? sp : p)
+	{
+	  case DMARC_RECORD_P_QUARANTINE:
+		apolicy = "quarantine";
+		break;
+
+	  case DMARC_RECORD_P_REJECT:
+		apolicy = "reject";
+		break;
+
+	  case DMARC_RECORD_P_UNSPECIFIED:
+	  case DMARC_RECORD_P_NONE:
+	  default:
+		apolicy = "none";
+		break;
+	}
+
 	/*
 	**  Generate a failure report.
 	*/
@@ -2726,6 +2802,13 @@ mlfi_eom(SMFICTX *ctx)
 			if (strncasecmp(ruv[c], "mailto:", 7) != 0)
 				continue;
 
+			bang = strchr(ruv[c], '!');
+			if (bang != NULL)
+				*bang = '\0';
+
+			if (ruv[c][7] == '\0')
+				continue;
+
 			if (first)
 			{
 				dmarcf_dstring_cat(dfc->mctx_afrf, "To: ");
@@ -2845,8 +2928,9 @@ mlfi_eom(SMFICTX *ctx)
 			                      dfc->mctx_envfrom);
 
 			dmarcf_dstring_printf(dfc->mctx_afrf,
-			                      "Source-IP: %s\n",
-			                      cc->cctx_ipstr);
+			                      "Source-IP: %s (%s)\n",
+			                      cc->cctx_ipstr,
+			                      cc->cctx_host);
 
 			dmarcf_dstring_printf(dfc->mctx_afrf,
 			                      "Reported-Domain: %s\n\n",
@@ -2924,12 +3008,17 @@ mlfi_eom(SMFICTX *ctx)
 	{
 	  case DMARC_POLICY_ABSENT:		/* No DMARC record found */
 	  case DMARC_FROM_DOMAIN_ABSENT:	/* No From: domain */
-	  case DMARC_POLICY_NONE:		/* Accept and report */
 		aresult = "none";
 		ret = SMFIS_ACCEPT;
 		result = DMARC_RESULT_ACCEPT;
 		break;
 
+	  case DMARC_POLICY_NONE:		/* Alignment failed, but policy is none: */
+		aresult = "fail";		/* Accept and report */
+		ret = SMFIS_ACCEPT;
+		result = DMARC_RESULT_ACCEPT;
+		break;
+
 	  case DMARC_POLICY_PASS:		/* Explicit accept */
 		aresult = "pass";
 		ret = SMFIS_ACCEPT;
@@ -3007,6 +3096,22 @@ mlfi_eom(SMFICTX *ctx)
 		break;
 	}
 
+	/* prepare human readable dispositon string for later processing */
+	switch (result)
+	{
+	  case DMARC_RESULT_REJECT:
+		adisposition = "reject";
+		break;
+
+	  case DMARC_RESULT_QUARANTINE:
+		adisposition = "quarantine";
+		break;
+
+	  default:
+		adisposition = "none";
+		break;
+	}
+
 	if (conf->conf_dolog)
 	{
 		syslog(LOG_INFO, "%s: %s %s", dfc->mctx_jobid,
@@ -3017,11 +3122,11 @@ mlfi_eom(SMFICTX *ctx)
 	if (ret != SMFIS_TEMPFAIL && ret != SMFIS_REJECT)
 	{
 		snprintf(header, sizeof header,
-		         "%s%s%s; dmarc=%s header.from=%s",
+		         "%s%s%s; dmarc=%s (p=%s dis=%s) header.from=%s",
 		         authservid,
 		         conf->conf_authservidwithjobid ? "/" : "",
 		         conf->conf_authservidwithjobid ? dfc->mctx_jobid : "",
-		         aresult, dfc->mctx_fromdomain);
+		         aresult, apolicy, adisposition, dfc->mctx_fromdomain);
 
 		if (dmarcf_insheader(ctx, 1, AUTHRESULTSHDR,
 		                     header) == MI_FAILURE)
